/*== chmp.js ===========================================================
Swiss member of parliament mail addresses.
Version     : $Id: cmd.js 477 2010-06-24 11:35:45Z hartwig $
Application : Open Government Data
Description : JScript module for retrieving national council mail addresses.
------------------------------------------------------------------------
Created    : 13.04.2011, Hartwig Thomas, Enter AG, Zurich
Copyright (c) 2011, Enter AG, Zurich, Switzerland
This file is part of the project Bundesversammlung.
chmp.js is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 2 of the License, or
any later version.
chmp.js is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with Bundesversammlung. If not, see <http://www.gnu.org/licenses/>.
======================================================================*/
/* Help avoid duplicate import */
var g_chmpjs = true;
/* Report loading */
if (g_bVerbose)
  WScript.StdOut.WriteLine("loading chmp.js");
  
/*======================================================================
ChMp
======================================================================*/
/* console i/o */
if (typeof g_ciojs == "undefined") eval(g_fso.OpenTextFile(getScriptDir() + "cio.js", 1).ReadAll());
/* http is needed for retrieving web pages */
if (typeof g_httpjs == "undefined") eval(g_fso.OpenTextFile(getScriptDir()+"http.js",1).ReadAll());
/* string is needed for trimming strings */
if (typeof g_stringjs == "undefined") eval(g_fso.OpenTextFile(getScriptDir()+"string.js",1).ReadAll());
  
/*----------------------------------------------------------------------
Main Page constants
----------------------------------------------------------------------*/
ChMp.sURL_NATIONAL_COUNCIL = "http://www.parlament.ch/e/organe-mitglieder/nationalrat/mitglieder-nr-a-z/Pages/default.aspx";
ChMp.sURL_COUNCIL_STATES = "http://www.parlament.ch/e/organe-mitglieder/staenderat/miglieder-sr-a-z/Pages/default.aspx";

/*----------------------------------------------------------------------
ChMp class can be initialized for the National Council or the Council of States
----------------------------------------------------------------------*/
function ChMp(sUrlMain)
{
  this.http = new Http();
  this.sCouncil = null;
  if (sUrlMain == ChMp.sURL_NATIONAL_COUNCIL)
    this.sCouncil = "National Council";
  if (sUrlMain == ChMp.sURL_COUNCIL_STATES)
    this.sCouncil = "Council of States";
  if (this.sCouncil != null)
  {
    var sMainPage = this.http.get(sUrlMain);
    /* now parse it for biografie_id=<nnn> */
    var re = new RegExp("^.*biografie_id=(\\d*).*$","gm");
    /* this yields a list of individual URLs for each biography */
    this.asUrl = sMainPage.match(re);
    if (this.asUrl != null)
    {
      for (var i = 0; i < this.asUrl.length; i++)
        this.asUrl[i] = this.asUrl[i].replace(re,"http://www.parlament.ch/e/suche/pages/biografie.aspx?biografie_id=$1");
    }
  }
}; /* ChMp */

/*----------------------------------------------------------------------
getMailAddress returns all mail addresses from the biography URL in 
the form "John Smith <john.smith@example.com>".
----------------------------------------------------------------------*/
ChMp.prototype.getMailAddress = function(sUrl)
{
  g_log.enter("getMailAddress", "sUrl");
  var asMailAddress = [];
  var sName = null;
  var sEmail = null;
  sBiographyPage = this.http.get(sUrl);
  /* <h1> contains either Rapid Access or name */
  var re = new RegExp("<h1>([^<]*)</h1>","g");
  var as = sBiographyPage.match(re);
  if (as != null)
  {
    for (var i = 0; i < as.length; i++)
    {
      as[i] = as[i].replace(re,"$1");
      var s = as[i].trim();
      if (s != "Rapid access")
        sName = s;
    }
    g_log.event("Name: "+sName);
  }
  /* there may be more than one <a href="mailto:fabio@abatelegal.ch"> */
  re = new RegExp("<a[^>]*href=\"mailto:(.*@.*)\">","g");
  as = sBiographyPage.match(re);
  if (as != null)
  {
    for (var i = 0; i < as.length; i++)
    {
      sEmail = as[i].replace(re,"$1");
      asMailAddress[i] = sName+" <"+sEmail+">";
      Cio.outln("  "+asMailAddress[i]);
    }
  }
  g_log.exit("getMailAddress", asMailAddress);
  return asMailAddress;
} /* getMailAddress */

/*----------------------------------------------------------------------
getMailAddresses returns a list of mail addresses in the form
"John Smith <john.smith@example.com>" separated by the given separator.
If bAll is true, all mail addresses for each person are returned.
----------------------------------------------------------------------*/
ChMp.prototype.getMailAddresses = function(sSeparator,bAll)
{
  Cio.outln(this.sCouncil);
  var sMailAddresses = "";
  for (var i = 0; i < this.asUrl.length; i++)
  {
    var asMailAddress = this.getMailAddress(this.asUrl[i]);
    if (asMailAddress.length > 0)
    {
      if (sMailAddresses.length > 0)
        sMailAddresses = sMailAddresses + sSeparator;
      sMailAddresses = sMailAddresses + asMailAddress[0];
    }
    if (bAll)
    {
      for (var k = 1; k < asMailAddress.length; k++)
      {
        sMailAddresses = sMailAddresses + sSeparator;
        sMailAddresses = sMailAddresses + asMailAddress[k];
      }
    }
  }
  return sMailAddresses;
}; /* getMailAddresses */
