/*== cmd.js ============================================================
Command execution.
Version     : $Id: cmd.js 477 2010-06-24 11:35:45Z hartwig $
Application : Utilities
Description : JScript module for command execution.
------------------------------------------------------------------------
Created    : 04.08.2008, Hartwig Thomas, Enter AG, Zurich
Copyright (c) 2009, Enter AG, Zurich, Switzerland, 2009
This file is part of the project Bundesversammlung.
cmd.js is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 2 of the License, or
any later version.
cmd.js is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with Bundesversammlung. If not, see <http://www.gnu.org/licenses/>.
======================================================================*/
/* Help avoid duplicate import */
var g_cmdjs = true;
/* Report loading */
if (g_bVerbose)
  WScript.StdOut.WriteLine("loading cmd.js");
  
/*======================================================================
Cmd
======================================================================*/
/* cio is needed for formatting timestamp of error messages */
if (typeof g_ciojs == "undefined")
  eval(g_fso.OpenTextFile(getScriptDir()+"cio.js",1).ReadAll());
  
/*----------------------------------------------------------------------
Cmd class
----------------------------------------------------------------------*/
function Cmd()
{
  this.sOutput = null;
  this.sError = null;
  this.shell = WScript.CreateObject("WScript.Shell");
}; /* Cmd */

/*----------------------------------------------------------------------
nowms current Date/Time im Milliseconds
----------------------------------------------------------------------*/
function nowms()
{
    return new Date().getTime();
}; /* nowms */

/*----------------------------------------------------------------------
(public) run
 run uses WScript.shell.Exec, and Wait for running a command
 and returns the exit code. If requested, the contents of StdOut and StdErr
 are piped through or saved in this.sOutput/this.sError.
Parameters:
  iOuputLength if < 0, then child output disappears,
               if = 0 then child output is piped through to stdout,
               if > 0 then the last iOutputLength bytes of output are
               stored in this.sOutput.
  iErrorLength ditto.
----------------------------------------------------------------------*/
Cmd.prototype.run = function(sCommand, iOutputLength, iErrorLength)
{
  g_log.enter("Cmd.prototype.run",sCommand+", "+iOutputLength+", "+iErrorLength);
  this.sOutput = "";
  this.sError = "";
  var iMsSleep = 100; /* check stdout and stderr all 100 ms */
 var iReturn = 0;
  /* execute the command */
  var wse = this.shell.Exec(sCommand);
  /* do a not terribly busy loop, waiting for completion */
  while (wse.Status == 0)
  {
    /* sleep 100 ms */
    WScript.Sleep(iMsSleep);
    /* if there is output then read it */
    if (!wse.StdOut.AtEndOfStream)
    {
      for (var sChar = wse.StdOut.Read(1); 
           (sChar != null) && (sChar.length > 0); 
           sChar = wse.StdOut.Read(1))
      {
        if (iOutputLength == 0)
          Cio.out(sChar);
        else if (iOutputLength > 0)
        {
          if (this.sOutput.length < iOutputLength)
            this.sOutput = this.sOutput + sChar;
          else
            this.sOutput = this.sOutput.substring(1)+sChar;
        }
      }
    }
    /* if there is error output then read it */
    if (!wse.StdErr.AtEndOfStream)
    {
      for (var sChar = wse.StdErr.Read(1); 
           (sChar != null) && (sChar.length > 0); 
           sChar = wse.StdErr.Read(1))
      {
        if (iErrorLength == 0)
          Cio.err(sChar);
        else if (iErrorLength > 0)
        {
          if (this.sError.length < iErrorLength)
            this.sError = this.sError + sChar;
          else
            this.sError = this.sError.substring(1)+sChar;
        }
      }
    }
  } /* while wse.Status == 0 */
  iReturn = wse.ExitCode;
  g_log.exit("Cmd.prototype.run",String(iReturn));
  return iReturn;

}; /* run */
