/*== fsu.js ============================================================
File system utilities.
Version     : $Id: fsu.js 477 2010-06-24 11:35:45Z hartwig $
Application : Utilities
Description : JScript module for file system utilities based on the
              global FileSystemObject g_fso
              (http://msdn.microsoft.com/en-us/library/6kxy1a51(VS.85).aspx)
------------------------------------------------------------------------
Created    : 04.08.2008, Hartwig Thomas, Enter AG, Zurich
Copyright (c) 2009, Enter AG, Zurich, Switzerland
This file is part of the project Bundesversammlung.
fsu.js is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 2 of the License, or
any later version.
fsu.js is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with Bundesversammlung. If not, see <http://www.gnu.org/licenses/>.
======================================================================*/
/* Help avoid duplicate import */
var g_fsujs = true;
/* Report loading */
if (g_bVerbose)
    WScript.StdOut.WriteLine("loading fsu.js");

/*======================================================================
Fsu
======================================================================*/
/* cmd is needed for executing plink.exe */
if (typeof g_cmdjs == "undefined")
  eval(g_fso.OpenTextFile(getScriptDir()+"cmd.js",1).ReadAll());

/*----------------------------------------------------------------------
Fsu class
----------------------------------------------------------------------*/
function Fsu()
{
}; /* Fsu */

/*----------------------------------------------------------------------
DriveType constants
----------------------------------------------------------------------*/
Fsu.iDRIVETYPE_NETWORK = 3;

/*----------------------------------------------------------------------
(public static) getDocumentsFolder returns the current user's 
"My Documents" folder.
----------------------------------------------------------------------*/
Fsu.getDocumentsFolder = function()
{
  var shell = new ActiveXObject("WScript.Shell");
  var sDocumentsFolder = shell.SpecialFolders("MyDocuments");
  /* if drive is mapped then return "absolute" network share */
  var folderDocuments = g_fso.getFolder(sDocumentsFolder);
  var driveDocuments = folderDocuments.Drive;
  if ((sDocumentsFolder.charAt(1) == ":") &&
    (driveDocuments.DriveType == Fsu.iDRIVETYPE_NETWORK))
    sDocumentsFolder = driveDocuments.ShareName & sDocumentsFolder.substr(2);
  return Fsu.folderPath(sDocumentsFolder);
};  /* getDocumentsFolder */

/*----------------------------------------------------------------------
(public static) folderPath makes sure, given path terminates in a
backslash.
Parameter sFolder must not be null or undefined.
----------------------------------------------------------------------*/
Fsu.folderPath = function(sFolder)
{
  /* all folders must have a trailing slash */
  if (sFolder.substring(sFolder.length-1) != "\\")
    sFolder = sFolder + "\\";
  return sFolder;
}; /* folderPath */

/*----------------------------------------------------------------------
(public static) folderParent retrieves the parent folder 
----------------------------------------------------------------------*/
Fsu.folderParent = function(sFolder)
{
  var sParentFolder = g_fso.GetParentFolderName(sFolder);
  sParentFolder = Fsu.folderPath(sParentFolder);
  return sParentFolder;
}; /* folderParent */

/*----------------------------------------------------------------------
(public static) folderTemp retrieves the local temp folder 
----------------------------------------------------------------------*/
Fsu.folderTemp = function()
{
  var iTemporaryFolder = 2;
  var sTempFolder = g_fso.GetSpecialFolder(iTemporaryFolder) + "\\";
  return sTempFolder;
}; /* folderTemp */

/*----------------------------------------------------------------------
(public static) fileTemp generates a temporary file name in the folder 
----------------------------------------------------------------------*/
Fsu.fileTemp = function(sFolder)
{
  var sFilename = sFolder + g_fso.GetTempName();
  return sFilename;
}; /* fileTemp */

/*----------------------------------------------------------------------
(public static) folderExists, returns true, if folder exists.
----------------------------------------------------------------------*/
Fsu.folderExists = function(sFolder)
{
  return g_fso.FolderExists(sFolder);
}; /* folderExists */

/*----------------------------------------------------------------------
(public static) fileExists, returns true, if file exists.
----------------------------------------------------------------------*/
Fsu.fileExists = function(sFile)
{
  return g_fso.FileExists(sFile);
}; /* fileExists */

/*----------------------------------------------------------------------
(public static) executableExists, returns true, if executable is available.
----------------------------------------------------------------------*/
Fsu.executableExists = function(sExecutable)
{
  var cmd = new Cmd();
  var iResult = cmd.run("cmd /C for %f in (" + sExecutable + ") do if (%~$PATH:f)==() exit 1 else exit 0",0,0);
  return (iResult == 0);
}; /* executableExists */

/*----------------------------------------------------------------------
(public static) isAbsolute 
----------------------------------------------------------------------*/
Fsu.isAbsolute = function(sPath)
{
  var bAbsolute = false;
  if (sPath)
  {
    if (sPath.charAt(0) == "\\")
      bAbsolute = true;
    else if ((sPath.length > 2) && (sPath.charAt(1) == ":") && (sPath.charAt(2) == "\\"))
      bAbsolute = true;
  }
  return bAbsolute;
}; /* isAbsolute */

/*----------------------------------------------------------------------
(public static) makeAbsolute 
----------------------------------------------------------------------*/
Fsu.makeAbsolute = function(sRelative)
{
  var sAbsolute = g_fso.GetAbsolutePathName(sRelative); 
  if (sRelative.charAt(sRelative.length-1) == "\\")
    sAbsolute = sAbsolute + "\\";
  return sAbsolute;
}; /* makeAbsolute */

/*----------------------------------------------------------------------
(public static) getExtension 
----------------------------------------------------------------------*/
Fsu.getExtension = function(sFilename)
{
  var sExtension = g_fso.GetExtensionName(sFilename); 
  return sExtension;
}; /* getExtension */

/*----------------------------------------------------------------------
(public static) deleteFile deletes the file(s) (file name may contain
wildcard characters)
If an error occurred (i.e. no files to delete or access denied), false
is returned.
----------------------------------------------------------------------*/
Fsu.deleteFile = function(sFileName,bForce)
{
  g_log.enter("Fsu.deleteFile",sFileName+", "+bForce);
  var bDeleted = false;
  try
  {
    g_fso.deleteFile(sFileName,bForce);
    bDeleted = true;
  }
  catch(e) { }
  g_log.exit("Fsu.deleteFile",String(bDeleted));
  return bDeleted;
}; /* deleteFile */

/*----------------------------------------------------------------------
(public static) deleteFile deletes the file(s) (file name may contain
wildcard characters)
If an error occurred (i.e. no files to delete or access denied), false
is returned.
----------------------------------------------------------------------*/
Fsu.moveFile = function(sFromFile,sToFile)
{
  g_log.enter("Fsu.moveFile",sFromFile+", "+sToFile);
  var bMoved = false;
  try
  {
    g_fso.MoveFile(sFromFile,sToFile);
    bMoved = true;
  }
  catch(e) { }
  g_log.exit("Fsu.moveFile",String(bMoved));
  return bMoved;
}; /* moveFile */

/*----------------------------------------------------------------------
(public static) createFolder creates the folder (recursively), if it does
not exist.
----------------------------------------------------------------------*/
Fsu.createFolder = function(sFolder)
{
  g_log.enter("Fsu.createFolder",sFolder);
  var bCreated = true;
  if (!g_fso.FolderExists(sFolder))
  {
    var sParentFolder = Fsu.folderParent(sFolder);
    if (!Fsu.folderExists(sParentFolder))
    {
      bCreated = Fsu.createFolder(sParentFolder);
    }
    if (bCreated)
    {
      bCreated = false;
      try
      {
        g_fso.CreateFolder(sFolder);
        bCreated = true;
      }
      catch(e) {}
    }
  }
  g_log.exit("Fsu.createFolder",String(bCreated));
  return bCreated;
}; /* createFolder */

/*----------------------------------------------------------------------
(public static) deleteFolder deletes the folder and all its contents.
----------------------------------------------------------------------*/
Fsu.deleteFolder = function(sFolder,bForce)
{
  g_log.enter("Fsu.deleteFolder",sFolder+", "+bForce);
  var bDeleted = false;
  try 
  {
    if (sFolder.substring(sFolder.length - 1) == "\\")
      sFolder = sFolder.substring(0,sFolder.length - 1);
    g_fso.DeleteFolder(sFolder, bForce);
    bDeleted = true;
  }
  catch(e) { }
  g_log.exit("Fsu.deleteFolder",String(bDeleted));
  return bDeleted;
}; /* deleteFolder */

/*----------------------------------------------------------------------
(public static) copyFolder copies source to target folder recursively.
----------------------------------------------------------------------*/
Fsu.copyFolder = function(sSourceFolder,sTargetFolder,bForce)
{
  g_log.enter("Fsu.copyFolder",sSourceFolder+", "+sTargetFolder+", "+bForce);
  var bCopied = false;
  try 
  {
    /* copy all folders */
    g_fso.CopyFolder(sSourceFolder+"*",sTargetFolder,bForce);
    /* copy all files */
    g_fso.CopyFile(sSourceFolder+"*",sTargetFolder,bForce);
    bCopied = true;
  }
  catch(e) { Cio.outln(e.name+": "+e.message); }
  g_log.exit("Fsu.copyFolder",String(bCopied));
  return bCopied;
}; /* copyFolder */

/*----------------------------------------------------------------------
ARCHIVE BIT in file attributes
----------------------------------------------------------------------*/
Fsu.iARCHIVE = 32;

/*----------------------------------------------------------------------
(public static) compareFiles compares two files using the Windows
  command fc.
----------------------------------------------------------------------*/
Fsu.compareFiles = function(file1,file2)
{
  var iAttributes = file1.Attributes;
  if (file1.Attributes == 0)
    file1.Attributes = Fsu.iARCHIVE;
  sCommand = "fc /b \"" + file1.Path + "\" \"" + file2.Path + "\"";
  var cmd = new Cmd();
  var iOutputLength = -1;
  if (g_log.bDebug)
    iOutputLength = 0;
  var iResult = cmd.run(sCommand,iOutputLength,0);
  if (iAttributes == 0)
    file1.Attributes = 0;
  return (iResult == 0);
}; /* compareFiles */

/*----------------------------------------------------------------------
(public static) compareFolders compares two folders recursively
----------------------------------------------------------------------*/
Fsu.compareFolders = function(folder1,folder2)
{
  var bEqual = true;
  /* files */
  if (bEqual)
  {
    var files1 = folder1.Files;
    var iCount1 = files1.Count;
    var files2 = folder2.Files;
    var iCount2 = files2.Count;
    if (iCount1 == iCount2)
    {
      enumFile1 = new Enumerator(files1);
      for (; bEqual && (!enumFile1.atEnd()); enumFile1.moveNext())
      {
        var file1 = enumFile1.item();
        var sFile2 = folder2.Path + "\\" + file1.Name;
        if (g_fso.FileExists(sFile2))
        {
          var file2 = g_fso.GetFile(sFile2);
          bEqual = Fsu.compareFiles(file1,file2);
        }
        else
        {
          Cio.outln("File \"" + file1.Name + "\" does not exist in \"" + folder2.Path + "!");
          bEqual = false;
        }
      }
    }
    else
    {
      Cio.outln("folders \"" + folder1.Path + "\" and \"" + folder2.Path + "\" contain an different number of files!");
      bEqual = false;
    }
  }
  /* sub folders */
  if (bEqual)
  {
    var foldersSub1 = folder1.SubFolders;
    var iCount1 = foldersSub1.Count;
    var foldersSub2 = folder2.SubFolders;
    var iCount2 = foldersSub2.Count;
    if (iCount1 == iCount2)
    {
      enumFolder1 = new Enumerator(foldersSub1);
      for (; bEqual && (!enumFolder1.atEnd()); enumFolder1.moveNext())
      {
        var folderSub1 = enumFolder1.item();
        var sSubFolder2 = folder2.Path + "\\" + folderSub1.Name;
        if (g_fso.FolderExists(sSubFolder2))
        {
          var folderSub2 = g_fso.GetFolder(sSubFolder2);
          bEqual = Fsu.compareFolders(folderSub1,folderSub2);
        }
        else
        {
          Cio.outln("Folder \"" + folderSub1.Name + "\" does not exist in \"" + folder2.Path + "!");
          bEqual = false;
        }
      }
    }
    else
    {
      Cio.outln("folders \"" + folder1.Path + "\" and \"" + folder2.Path + "\" contain an different number of subfolders!");
      bEqual = false;
    }
  }
  return bEqual;
}; /* compareFolders */

/*----------------------------------------------------------------------
(public static) areFoldersEqual checks, if two folders are equal.
----------------------------------------------------------------------*/
Fsu.areFoldersEqual = function(sFolderName1,sFolderName2)
{
  return Fsu.compareFolders(g_fso.GetFolder(sFolderName1),g_fso.GetFolder(sFolderName2));
}; /* areFoldersEqual */
